# -*- coding: utf-8 -*-

import requests
import json
from .exceptions import CyberarkException, GetTokenException, BastionException
from .config import Config
from .accounts import Account
from .platforms import Platform
from .abstract import Bastion


class EPV(Bastion):
    def __init__(self, configfile: str = None, username=None, password=None, serialized: dict = None):
        if configfile is None and serialized is None:
            raise BastionException("You must provide either configfile or serialized to init EPV")
        if configfile is not None and serialized is not None:
            raise BastionException("You must provide either configfile or serialized to init EPV, not both")
        if configfile is not None:
            self.config = Config(configfile)
            self.api_host = self.config.PVWA
            self.reqparms = {"timeout": self.config.timeout, "verify": self.config.CA}
            self.api_host = self.config.PVWA
            self.cpm = self.config.CPM
            self.retention = self.config.retention
            self.__token = None
            if username is not None:
                self.username = username
            else:
                self.username = self.config.username
            if password is not None:
                self.__password = password
            else:
                self.__password = self.config.password
            self.authtype = self.config.authtype

        if serialized is not None:
            self.reqparms = {"timeout": serialized["timeout"], "verify": serialized["verify"]}
            self.api_host = serialized['api_host']
            self.cpm = serialized['cpm']
            self.retention = serialized['retention']
            self.__token = serialized['token']
            self.authtype = serialized["authtype"]

        self.session = requests.Session()
        self.user_list = None

        # utilities
        self.account = Account(self)
        self.platform = Platform(self)

    # start of functions definition
    def __login_cyberark(self, username: str, password: str, authtype: str = "CyberArk") -> str:
        assert self.__token is None
        assert authtype in ("Cyberark", "Windows", "LDAP", "RADIUS")
        url, head = self.get_url("API/Auth/" + authtype + "/Logon")
        request_data = {"username": username, "password": password}
        try:
            req = self.session.post(url, headers=head, data=json.dumps(request_data), **self.reqparms)
            # Cleaning password after authentication
            self.__password = ""
        except ConnectionError or TimeoutError:
            raise CyberarkException("Network problem connecting to PVWA")
        if req.status_code != 200:
            raise CyberarkException("Invalid credentials ! " + str(req.status_code))
        return json.loads(req.content)

    def logoff(self):
        url, head = self.get_url("API/Auth/CyberArk/Logoff")
        req = self.session.post(url, headers=head, **self.reqparms)
        if req.status_code != 201:
            raise CyberarkException("Error disconnecting to PVWA")
            pass
        self.__token = None

    def __check_token(self) -> bool:
        if self.__token is None:
            return False
        url, head = self.get_url("WebServices/PIMServices.svc/User")
        req = self.session.get(url, headers=head, **self.reqparms)
        if req.status_code != 200:
            self.__token = None
            return False
        return True


    def login(self):
        if self.__check_token():
            return
        try:
            #TODO add in aim
            #account = self.get_password_object(self.__object)
            #user = account['UserName']
            #password = account['Content']
            user = self.username
            password = self.__password
        except PermissionError:
            raise GetTokenException("I was unable to grab password")
        except OSError:
            raise GetTokenException("i was unable to read the tmp file ")

        try:
            self.__token = self.__login_cyberark(user, password, self.authtype)
        except CyberarkException as err:
            raise GetTokenException(err)

    def get_url(self, url) -> (str, str):
        addr = 'https://' + self.api_host + '/PasswordVault/' + url
        head = {"Content-type": "application/json", "Authorization": self.__token}
        return addr, head

    def to_json(self):
        serialized = {
            "api_host": self.config.PVWA,
            "timeout": self.config.timeout,
            "verify": self.config.CA,
            "cpm": self.config.CPM,
            "retention": self.config.retention,
            "token": self.__token,
            "authtype": self.authtype
        }
        return serialized


